// SPDX-FileCopyrightText: 2021 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "logviewerplugininterface.h"

LogViewerPluginInterface::LogViewerPluginInterface()
{
}

LogViewerPluginInterface::~LogViewerPluginInterface()
{
}
