
#include "DocxFactory/DocxMerger/DocxMergerPasteChartAxis.h"

using namespace DocxFactory;
using namespace std;



DocxMergerPasteChartAxis::DocxMergerPasteChartAxis()
{

} // c'tor

DocxMergerPasteChartAxis::~DocxMergerPasteChartAxis()
{

} // d'tor
